
# $Id: LinuxIpfw.pm,v 1.3 1998/12/18 22:16:57 gtaylor Exp $

package Autobuse::LinuxIpfw;

use Autobuse::Plugin;
use Autobuse::Common;
use Autobuse::Event;

push (@ISA, Autobuse::Plugin);	# inherit base plugin methods

# return a hash with our configuration
sub new {
    my %this = ('type' => $_[0],
		'external_host' => $_[1],
		'external_iface' => $_[2],
		'ignore_domains' => $_[3],
		'logtype' => "syslog");
    return bless \%this;
}

# check a line against our rules
# we return undef or an Event object
sub check_logline {
    my $this = $_[0];
    my $line = $_[1];
    
    # Default return hash; no results
    my $retval = undef;
    
    # Various handy local copies and refs
    my $external_host = $this->{'external_host'};
    my $external_iface = $this->{'external_iface'};
    my @ignore_domains = @{$this->{'ignore_domains'}};
    my $badports = get_badports();
    
    # This script can so far only handle ipfw lines
    if ($line =~ /([JFMASOND][a-z][a-z])\s+(\d+)\s+(\d\d:\d\d:\d\d)\s+(\w+) kernel: IP fw\-(in|out|forw) (deny|rej|acc) (eth\d) (TCP|UDP|ICMP)(.*)$/o) {
	
	my ($month, $date, $time, 
	    $host, $ruleset, $action, 
	    $iface, $protocol, $rest) = ($1, $2, $3, $4, $5, $6, $7, $8, $9);
	
	return $retval if ($host ne $external_host);
	return $retval if ($iface ne $external_iface);
	
	if ($protocol eq "TCP" || $protocol eq "UDP") {
	    $rest =~ /([\d\.]+):(\d+)\ ([\d\.]+):(\d+)/o;
	    my ($source_ip, $source_port, 
		$dest_ip, $dest_port) = ($1, $2, $3, $4);
	    
	    return $retval if ($dest_ip =~ /255$/); # don't sweat broadcasts
	    return $retval if ($dest_ip =~ /^224/); # don't sweat mcast pkts
	    
	    if (defined($badports->{$protocol}{$dest_port})) {
		
		# Get a nice string telling what this is
		my $what = $badports->{$protocol}{$dest_port};
		my $n = "";
		$n = "n" if ($what =~ /^[aeiou]/i);
		
		# If we have a hostname, be paranoid and then compare
		# against domain names specified in @ignore_domains.
		return $retval 
		    if paranoid_check_domain($source_ip, \@ignore_domains);

		# Report an Event
		$retval = new Autobuse::Event($source_ip,
					      "LinuxIpFw/$protocol:$dest_port",
					      "a$n $what",
					      $line);
	    }
	}
	
    } else {
	# ICMP
	
    }

    return $retval;

} 

1;

