
# Autobuse module implementing check for exploit attempts on Apache
# httpd - phf, etc

# $Id: Apache.pm,v 1.3 1998/12/18 22:16:56 gtaylor Exp $

package Autobuse::Apache;

use Autobuse::Plugin;
push (@ISA, Autobuse::Plugin);	# inherit base plugin methods

use Autobuse::Common;		# utility functions
use Autobuse::Event;		# return type

my $badurls = get_badurls();

# initialize this filter
# arg 1 = "common" (traditional apache, etc) or "combined" (a type in apache 1.3.x)
sub new {
    my %this = ('type' => $_[0],
		'format' => $_[1],
		'ignore_domains' => $_[2],
		'logtype' => "web");
    return bless \%this;
}

# check a line against our rules
# we return an Event or undef
sub check_logline {
    my $this = $_[0];
    my $line = $_[1];
    
    # Default return hash; no results
    my $retval = undef;

    my $ignore_domains = $this->{'ignore_domains'};
    my $format = $this->{'format'};

    # The two formats are the same for now...
    if ($format eq "common" || $format eq "combined") {
	for $badurl (@{$badurls}) {
	    if ($line =~ /^(\S+) .+\"GET $badurl/) {
		my $ip = $1;

		# Someone calling an exploitable script?
	    
		# skip if IP is in our list of trusted domains
		return $retval if paranoid_check_domain($ip, $ignore_domains);

		$retval = new Autobuse::Event($ip,
					      "Apache/$badurl",
					      "$badurl exploit attempt",
					      $line);
		return $retval;
	    }
	}
    }

    return $retval;
}


1;
